use strict;
use Time::HiRes                                       qw( time );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use File::Temp qw ( tempdir tempfile );
use Readonly;
use Test::More;
use Data::Dumper;

Readonly my $ROOTPATH => '/TestShop';

sub Main {## no critic (ProhibitFunctionSize)
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );
            my $StartTime;
            my $FinishingTime;
            # tests
            eval {
                  ok (my $PortalConfig = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShop', 'PortalConfigs', 'GoogleBase_de_DE' ] ) ), 'Load portal config');
                  isa_ok( $PortalConfig, 'DE_EPAGES::ProductPortal::API::Object::PortalConfigWithUpload' );
                  my $dir = tempdir( CLEANUP => 1 );
                  my (undef, $FileName) = tempfile( DIR => $dir );
                  $StartTime = time;
                  $PortalConfig->exportPortal($FileName, 1);
                  $FinishingTime = time;
                  note (sprintf('The Export needed %.4f seconds ', $FinishingTime - $StartTime));
                  ok (-f $FileName,'export file created');
                  #- 1Flasche-with refPrice---------------------------------------------------------------------
                  my $regEx = '\(1 Flasche\(n\)=34.00';
                  my @RegExResult = (${GetFileContent($FileName)} =~ m/$regEx/g);
                  is (scalar @RegExResult,2,'1 Flasche-with refPrice in title and Description');
                  #- Kaffee 1000g no refPrice------------------------------------------------------------------
                  $regEx = 'Kaffee 1000g_de_DE\s*Eureka';
                  @RegExResult = (${GetFileContent($FileName)} =~ m/$regEx/g);
                  is (scalar @RegExResult,1,'Kaffee 1000g RefContentAmount:1000 PriceQuantity:1 RefAmount:1000 no reference Price');
                  #- Kaffee 500g -with refPrice-----------------------------------------------------------------
                  $regEx = 'Kaffee 500g_de_DE\s*\(1.000 g=238.00';
                  @RegExResult = (${GetFileContent($FileName)} =~ m/$regEx/g);
                  is (scalar @RegExResult,1,'Kaffee 500g RefContentAmount:500 PriceQuantity:1 RefAmount:1000 with reference Price');
                  #- Gartenstuhl 1 1 1 no refPrice --------------------------------------------------------------
                  $regEx = 'Gartenstuhl 1 1 1_de_DE\s*Eureka';
                  @RegExResult = (${GetFileContent($FileName)} =~ m/$regEx/g);
                  is (scalar @RegExResult,1,'Gartenstuhl RefContentAmount:1 PriceQuantity:1 RefAmount:1 no refPrice');
                  #- Gartenstuhl 3 3 1 with refPrice --------------------------------------------------------------
                  $regEx = 'Gartenstuhl 3 3 1_de_DE\s*\(';
                  @RegExResult = (${GetFileContent($FileName)} =~ m/$regEx/g);
                  is (scalar @RegExResult,1,'Gartenstuhl RefContentAmount:3 PriceQuantity:3 RefAmount:1 with refPrice');
                  #- Gartenstuhl 3 3 3 no refPrice --------------------------------------------------------------
                  $regEx = 'Gartenstuhl 3 3 3_de_DE\s*Eureka';
                  @RegExResult = (${GetFileContent($FileName)} =~ m/$regEx/g);
                  is (scalar @RegExResult,1,'Gartenstuhl RefContentAmount:3 PriceQuantity:3 RefAmount:3 no refPrice');
                 };
             if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }
            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestShop' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Debug => 1, Sub => \&Main );

1;